<?php


namespace Schaffen\Core\Movement\Managers;


use Carbon\Carbon;
use Freelabois\LaravelQuickstart\Extendables\ManipulationManager;
use Schaffen\Core\Movement\Repositories\SalesItemRepository;
use Schaffen\Core\Movement\Repositories\SalesPaymentFormRepository;
use Schaffen\Core\Movement\Repositories\SalesRepository;
use Schaffen\Core\Movement\Services\SalesValidateService;
use Schaffen\WarehouseInventory\WarehouseInventory\Repositories\AccountInventoryRepository;
use Schaffen\WarehouseInventory\WarehouseInventory\Repositories\SnapshotInventoryRepository;

class SalesManager extends ManipulationManager
{
    protected $validation = [
        self::CREATE_MESSAGE => [
            "pes_codigo.required" => 'O código da pessoa é obrigatório',
            "pev_data_movel.required" => 'A data do pedido é obrigatório',
        ],
    ];

    protected $repository;

    public function __construct(
        SalesRepository $repository,
        SalesItemRepository $sales_item_repository,
        SalesPaymentFormRepository $sales_payment_form_repository,
        SalesValidateService $sales_validate_service,
        AccountInventoryRepository $account_inventory_repository,
        SnapshotInventoryRepository $snaphot_inventory_repository)
    {
        $this->repository = $repository;
        $this->sales_validate_service = $sales_validate_service;
        $this->sales_item_repository = $sales_item_repository;
        $this->sales_payment_form_repository = $sales_payment_form_repository;
        $this->account_inventory_repository = $account_inventory_repository;
        $this->snapshot_inventory_repository = $snaphot_inventory_repository;
    }

    public function ultimasVendas($values){
        return $this->repository->setOrder('ven_codigo', 'desc')->list(['ven_fk_pes_codigo_vendedor' => $values['usu_fk_pes_codigo']], ['client', 'sales_items'], 10);
    }

    public function addVenda($values)
    {
        $response = $this->sales_validate_service->validar_venda($values);

        if ($response['status_code'] == 500) {
            return $response;
        } else {
            $id = $values['ven_codigo'] ?? null;
            $venda = [];
            $venda['ven_fk_pes_codigo_vendedor'] = auth()->user()['usu_fk_pes_codigo'];
            $venda['ven_fk_fil_codigo'] = $values['fun_fk_fil_codigo'];
            $venda['ven_data_movel'] = $values['ven_data_movel'];
            $venda['ven_fk_pes_codigo'] = $values['pes_codigo'];
            $venda['ven_fk_loe_codigo'] = $values['loe_codigo'];
            $nova_venda = $this->repository->storeOrUpdate($venda, $id);

            $itens_da_venda = [];
            $itens_da_venda['ven_codigo'] = $nova_venda['ven_codigo'];
            $itens_da_venda['itens'] = $values['itens'];
            $this->sales_item_repository->storeOrUpdate($itens_da_venda);

            $conta_corrente = [];
            $conta_corrente['loe_codigo'] = $values['loe_codigo'];
            $conta_corrente['ven_codigo'] = $nova_venda['ven_codigo'];
            $conta_corrente['conta_corrente'] = $values['itens'];
            $this->account_inventory_repository->storeOrUpdateMobile($conta_corrente);

            $snapshot = [];
            $snapshot['snapshot'] = $values['itens'];
            $snapshot['loe_codigo'] = $values['loe_codigo'];
            $this->snapshot_inventory_repository->storeOrUpdateMobile($snapshot);

            $formas_pagamento = [];
            $formas_pagamento['ven_codigo'] = $nova_venda['ven_codigo'];
            $formas_pagamento['forma_pagamento'] = $values['forma_pagamento'];
            $this->sales_payment_form_repository->storeOrUpdate($formas_pagamento);
        }
        return $response;
    }

    public function cancelarVenda($values)
    {
        $venda = $this->repository->first(['ven_codigo' => $values['ven_codigo']]);
        if(!$venda->ven_data_cancelamento){
            $venda = $this->repository->storeOrUpdate(['ven_data_cancelamento' => Carbon::now()], $values['ven_codigo']);
        }
        return $venda;
    }
}
